/**
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.admin;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.Serializable;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.JCheckBox;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import sync4j.framework.engine.source.ContentType;
import sync4j.framework.engine.source.SyncSource;
import sync4j.framework.engine.source.SyncSourceInfo;

import sync4j.syncadmin.ui.ManagementPanel;

import sync4j.exchange.engine.source.ExchangeSyncSource;

import org.apache.commons.lang.StringUtils;

/**
 * This class implements the configuration panel for an ExchangeSyncSource
 *
 * @author Luigia Fassina
 *
 * @version $Id: ExchangeSyncSourceConfigPanel.java,v 1.3 2004/04/07 11:54:41 fabius Exp $
 */
public class ExchangeSyncSourceConfigPanel
extends ManagementPanel
implements Serializable {

  // ----------------------------------------------------------------- Constants

  public static final String NAME_ALLOWED_CHARS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890-_.";
  public static final String PORT_ALLOWED_CHARS = "0123456789";

  // -------------------------------------------------------------- Private data
  /** label for the panel's name */
  private JLabel panelName = new JLabel();

  /** border to evidence the title of the panel */
  private TitledBorder titledBorder1;

  private JLabel nameLabel = new JLabel();
  private JTextField nameValue = new JTextField();

  private JLabel typeLabel = new JLabel();
  private JTextField typeValue = new JTextField();

  private JLabel sourceUriLabel = new JLabel();
  private JTextField sourceUriValue = new JTextField();

  private JLabel hostLabel = new JLabel();
  private JTextField hostValue = new JTextField();

  private JLabel portLabel = new JLabel();
  private JTextField portValue = new JTextField();

  private JLabel tmzLabel = new JLabel();
  private JTextField tmzValue = new JTextField();

  private JLabel infoTypesLabel = new JLabel();
  private JTextField infoTypesValue = new JTextField();

  private JLabel infoVersionsLabel = new JLabel();
  private JTextField infoVersionsValue = new JTextField();

  private JLabel encodeLabel = new JLabel();
  private JCheckBox encodeValue = new JCheckBox();

  private JButton confirmButton = new JButton();


  private ExchangeSyncSource syncSource = null;

  // ------------------------------------------------------------ Constructors


  public ExchangeSyncSourceConfigPanel() {
      init();
  }


  // ----------------------------------------------------------- Private methods
  /**
   * Create the panel
   * @throws Exception if error occures during creation of the panel
   */

  private void init(){
    // set layout
    this.setLayout(null);

    // set properties of label, position and border
    //  referred to the title of the panel
    titledBorder1 = new TitledBorder("");

    panelName.setFont(titlePanelFont);
    panelName.setText("Edit Exchange SyncSource");
    panelName.setBounds(new Rectangle(14, 5, 316, 28));
    panelName.setAlignmentX(SwingConstants.CENTER);
    panelName.setBorder(titledBorder1);

    sourceUriLabel.setText("Source URI: ");
    sourceUriLabel.setFont(defaultFont);
    sourceUriLabel.setBounds(new Rectangle(14, 60, 150, 18));
    sourceUriValue.setFont(new java.awt.Font("Arial", 0, 12));
    sourceUriValue.setBounds(new Rectangle(170, 60, 350, 18));

    nameLabel.setText("Name: ");
    nameLabel.setFont(defaultFont);
    nameLabel.setBounds(new Rectangle(14, 90, 150, 18));
    nameValue.setFont(new java.awt.Font("Arial", 0, 12));
    nameValue.setBounds(new Rectangle(170, 90, 350, 18));

    typeLabel.setText("Type: ");
    typeLabel.setFont(defaultFont);
    typeLabel.setBounds(new Rectangle(14, 120, 150, 18));
    typeValue.setFont(new java.awt.Font("Arial", 0, 12));
    typeValue.setBounds(new Rectangle(170, 120, 350, 18));

    hostLabel.setText("Host: ");
    hostLabel.setFont(defaultFont);
    hostLabel.setBounds(new Rectangle(14, 150, 150, 18));
    hostValue.setFont(new java.awt.Font("Arial", 0, 12));
    hostValue.setBounds(new Rectangle(170, 150, 350, 18));

    portLabel.setText("Port: ");
    portLabel.setFont(defaultFont);
    portLabel.setBounds(new Rectangle(14, 180, 150, 18));
    portValue.setFont(new java.awt.Font("Arial", 0, 12));
    portValue.setBounds(new Rectangle(170, 180, 350, 18));

    tmzLabel.setText("Timezone: ");
    tmzLabel.setFont(defaultFont);
    tmzLabel.setBounds(new Rectangle(14, 210, 150, 18));
    tmzValue.setFont(new java.awt.Font("Arial", 0, 12));
    tmzValue.setBounds(new Rectangle(170, 210, 350, 18));

    infoTypesLabel.setText("Supported types: ");
    infoTypesLabel.setFont(defaultFont);
    infoTypesLabel.setBounds(new Rectangle(14, 240, 150, 18));
    infoTypesValue.setFont(new java.awt.Font("Arial", 0, 12));
    infoTypesValue.setBounds(new Rectangle(170, 240, 350, 18));

    infoVersionsLabel.setText("Supported versions: ");
    infoVersionsLabel.setFont(defaultFont);
    infoVersionsLabel.setBounds(new Rectangle(14, 270, 150, 18));
    infoVersionsValue.setFont(new java.awt.Font("Arial", 0, 12));
    infoVersionsValue.setBounds(new Rectangle(170, 270, 350, 18));

    encodeLabel.setText("Encoded: ");
    encodeLabel.setFont(defaultFont);
    encodeLabel.setBounds(new Rectangle(14, 300, 150, 18));
    encodeValue.setSelected(false);
    encodeValue.setBounds(167, 300, 50, 25);

    confirmButton.setFont(defaultFont);
    confirmButton.setText("Add");
    confirmButton.setBounds(170, 330, 70, 25);

    confirmButton.addActionListener(new ActionListener() {
        public void actionPerformed(ActionEvent event ) {
            try {
                validateValues();
                getValues();
                  if (setSyncSource(syncSource)) {
                      sourceUriValue.setEditable(false);
                      confirmButton.setText("Save");
                  }
            } catch (Exception e) {
                error(e.getMessage());
            }
        }
    });


    // add all components to the panel
    this.add(panelName        , null);
    this.add(nameLabel        , null);
    this.add(nameValue        , null);
    this.add(typeLabel        , null);
    this.add(typeValue        , null);
    this.add(hostLabel        , null);
    this.add(hostValue        , null);
    this.add(portLabel        , null);
    this.add(portValue        , null);
    this.add(tmzLabel         , null);
    this.add(tmzValue         , null);
    this.add(sourceUriLabel   , null);
    this.add(sourceUriValue   , null);
    this.add(infoTypesLabel   , null);
    this.add(infoTypesValue   , null);
    this.add(infoVersionsLabel, null);
    this.add(infoVersionsValue, null);
    this.add(encodeLabel      , null);
    this.add(encodeValue      , null);
    this.add(confirmButton    , null);

  }


  public void loadSyncSource(SyncSource syncSource) {
      if (!(syncSource instanceof ExchangeSyncSource)) {
          error("This is not a ExchangeSyncSource! Unable to process SyncSource values.");
          return;
      }

      if (getState() == ManagementPanel.STATE_INSERT) {
          confirmButton.setText("Add");
      } else if (getState() == ManagementPanel.STATE_UPDATE) {
          confirmButton.setText("Save");
      }

      this.syncSource = (ExchangeSyncSource)syncSource;

      sourceUriValue.setText(this.syncSource.getSourceURI()      );
      nameValue.setText     (this.syncSource.getName()           );
      typeValue.setText     (this.syncSource.getType()           );
      hostValue.setText     (this.syncSource.getHost()           );
      portValue.setText     ("" + this.syncSource.getPort()      );
      tmzValue.setText      (this.syncSource.getTmz()            );

      if (this.syncSource.isEncode())
          encodeValue.setSelected(true);
      else encodeValue.setSelected(false);

      SyncSourceInfo info = this.syncSource.getInfo();
      if (info != null) {
          ContentType[] types = info.getSupportedTypes();

          StringBuffer typesList = new StringBuffer(), versionsList = new StringBuffer();
          for (int i=0; ((types != null) && (i<types.length)); ++i) {
              if (typesList.length() > 0) {
                  typesList.append(',');
                  versionsList.append(',');
              }
              typesList.append(types[i].getType());
              versionsList.append(types[i].getVersion());
          }

          infoTypesValue.setText(typesList.toString());
          infoVersionsValue.setText(versionsList.toString());
      }

      if (this.syncSource.getSourceURI() != null) {
          sourceUriValue.setEditable(false);
      }
  }


  // ----------------------------------------------------------- Private methods

  /**
   * Checks if the values provided by the user are all valid. In caso of errors,
   * a IllegalArgumentException is thrown.
   *
   * @throws IllegalArgumentException if:
   *         <ul>
   *         <li>name, uri, type or directory are empty (null or zero-length)
   *         <li>the types list length does not match the versions list length
   *         </ul>
   */
  private void validateValues() throws IllegalArgumentException {
      String value = null;

      value = nameValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Name' cannot be empty. Please provide a SyncSource name.");
      }

      if (!StringUtils.containsOnly(value, NAME_ALLOWED_CHARS.toCharArray())) {
          throw new IllegalArgumentException("Only the following characters are allowed for field 'Name': \n" + NAME_ALLOWED_CHARS);
      }

      value = typeValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Type' cannot be empty. Please provide a SyncSource type.");
      }

      value = hostValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Host' cannot be empty. Please provide a SyncSource host.");
      }

      value = portValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Port' cannot be empty. Please provide a SyncSource port.");
      }

      if (!StringUtils.containsOnly(value, PORT_ALLOWED_CHARS.toCharArray())) {
          throw new IllegalArgumentException("Only the following characters are allowed for field 'Port': \n" + PORT_ALLOWED_CHARS);
      }

      value = tmzValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Timezone' cannot be empty. Please provide a SyncSource timezone.");
      }

      value = sourceUriValue.getText();
      if (StringUtils.isEmpty(value)) {
          throw new IllegalArgumentException("Field 'Source URI' cannot be empty. Please provide a SyncSource URI.");
      }

      int typesCount = new StringTokenizer(infoTypesValue.getText(), ",").countTokens();
      if (typesCount == 0) {
          throw new IllegalArgumentException("Field 'Supported types' cannot be empty. Please provide one or more supported types.");
      }
      if (typesCount != new StringTokenizer(infoVersionsValue.getText(), ",").countTokens()) {
          throw new IllegalArgumentException("Number of supported types does not match number of versions");
      }
  }

  /**
   * Set syncSource properties with the values provided by the user.
   */
  private void getValues() {
      StringTokenizer types    = new StringTokenizer(infoTypesValue.getText(),    ",");
      StringTokenizer versions = new StringTokenizer(infoVersionsValue.getText(), ",");

      syncSource.setSourceURI      (sourceUriValue.getText().trim());
      syncSource.setName           (nameValue.getText().trim()     );
      syncSource.setType           (typeValue.getText().trim()     );
      syncSource.setHost           (hostValue.getText().trim()     );
      syncSource.setPort           (Integer.parseInt(portValue.getText().trim()));
      syncSource.setTmz            (tmzValue.getText().trim()      );

      if (encodeValue.isSelected())
          syncSource.setEncode(true);
      else syncSource.setEncode(false);

      ContentType[] contentTypes= new ContentType[types.countTokens()];

      for(int i = 0; i<contentTypes.length; ++i) {
          contentTypes[i] = new ContentType(types.nextToken().trim(), versions.nextToken().trim());
      }

      syncSource.setInfo(new SyncSourceInfo(contentTypes, 0));
  }

  private void error(String msg) {
      JOptionPane.showMessageDialog(null, msg, "Error", JOptionPane.ERROR_MESSAGE);
  }

}