/**
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.calendar.model;

import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.StringTokenizer;

import sync4j.exchange.items.calendar.CalendarParseException;
import sync4j.exchange.xml.XmlParser;
import sync4j.exchange.xml.XmlParseException;

/**
 * @author Fabio Maggi @ Funambol
 *
 * @version $Id: Calendar.java,v 1.4 2004/05/07 10:21:24 fabius Exp $
 **/

public class Calendar {

    //--------------------------------------------------------------------- Constants

    private static final String EXCHANGE_OOF_STATE           = "OUTOFOFFICE"  ;
    private static final String PP_OOF_STATE                 = "OOF"          ;

    private static final String KEY_ITEM_CLIENT_CATEGORIES   = "CATEGORIES:"  ;
    private static final String KEY_ITEM_CLIENT_DESCRIPTION  = "DESCRIPTION:" ;
    private static final String KEY_ITEM_CLIENT_DTSTART      = "DTSTART:"     ;
    private static final String KEY_ITEM_CLIENT_DTEND        = "DTEND:"       ;
    private static final String KEY_ITEM_CLIENT_LOCATION     = "LOCATION:"    ;
    private static final String KEY_ITEM_CLIENT_SUMMARY      = "SUMMARY:"     ;

    private static final String TAG_PP_DATE_START            = "date-start"   ;
    private static final String TAG_PP_TIMEZONE_ID           = "timezone-id"  ;
    private static final String TAG_PP_DATE_END              = "date-end"     ;
    private static final String TAG_PP_DURATION              = "duration"     ;
    private static final String TAG_ALL_DAY_EVENT            = "all-day-event";
    private static final String TAG_PP_SUBJECT               = "subject"      ;
    private static final String TAG_PP_LOCATION              = "location"     ;
    private static final String TAG_PP_NOTE
           = "text-description"                                               ;
    private static final String TAG_PP_BUSY_STATUS           = "busy-status"  ;
    private static final String TAG_PP_MEETING_STATUS
           = "meeting-status"                                                 ;

    private static final String MSG_VCARD                =
            "BEGIN:VCALENDAR\n"                          +
            "VERSION:1.0\n"                              +
            "BEGIN:VEVENT\n"                             +
            "UID:1074800413959"                          +
            "DESCRIPTION:{0}\n"                          +
            "SUMMARY:{1}\n"                              +
            "DTSTART:{2}\n"                              +
            "DTEND:{3}\n"                                +
            "X-EPOCAGENDAENTRYTYPE:exchange\n"           +
            "CATEGORIES:exchange\n"                      +
            "LOCATION:{4}\n"                             +
            "END:VEVENT"                                 ;

    private static final String MSG_PP                   =
            "<![CDATA[\n"                                +
            "<subject>{0}</subject>\n"                   +
            "<location>{1}</location>\n"                 +
            "<text-description>{2}</text-description>\n" +
            "<busy-status>{3}</busy-status>\n"           +
            "<meeting-status>{4}</meeting-status>\n"     +
            "<timezone-id>{5}</timezone-id>\n"           +
            "<date-start>{6}</date-start>\n"             +
            "<date-end>{7}</date-end>\n"                 +
            "<duration>{8}</duration>\n"                 +
            "<all-day-event>{9}</all-day-event>"         +
            "]]>"                                        ;

    //--------------------------------------------------------------------- Private data

    private String     id                  = null  ;
    private String     username            = null  ;
    private String     subject             = null  ;
    private String     textDescription     = null  ;
    private String     location            = null  ;

    private Date       dateStart           = null  ;
    private Date       dateEnd             = null  ;
    private long       duration            = 0     ;
    private Date       createDate          = null  ;
    private Date       lastModifiedDate    = null  ;

    private String     busyStatus          = null  ;
    private String     meetingStatus       = null  ;

    private int        timezoneId          = 0     ;

    private int        allDayEvent         = 0     ;
    private int        sequence            = 0     ;

    private String     href                = null  ;

    private Date       lastModified        = null  ;
    private char       state               = ' '   ;

    //--------------------------------------------------------------------- Constructors

    /**
     * Creates a Calendar
     *
     */
    public Calendar () {

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     *
     */
    public Calendar (String id) {

        this.id = id   ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     * @param t client item timestamp
     *
     */
    public Calendar (String id, Date t)
    throws CalendarParseException {

        StringTokenizer  st             = null ;

        String           record         = null ;
        String           value          = null ;

        int              startPosition  = 0    ;
        int              endPosition    = 0    ;

        this.id            = id  ;
        this.lastModified  = t   ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     * @param state client item state
     *
     */
    public Calendar (String id, char state) {

        this.id      = id     ;
        this.state   = state  ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id
     * @param href
     * @param state
     * @param lastModified
     *
     */
    public Calendar (String     id    ,
                     String     href  ,
                     char       state ,
                     Date       lastModified) {

        this.id            = id            ;
        this.href          = href          ;
        this.state         = state         ;
        this.lastModified  = lastModified  ;

    }

    /**
     * Creates a Calendar
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public Calendar (String id, String content, Date t)
    throws CalendarParseException {

        getPPCalendar    (id, content, t);

    }

    //----------------------------------------------------------------------- Public methods

    public String getId()                  {
        return this.id;
    }

    public String getUsername()            {
            return this.username;
    }

    public String getSubject()             {
        return this.subject;
    }

    public String getTextDescription()     {
        return this.textDescription;
    }

    public int getTimezoneId()            {
        return this.timezoneId;
    }

    public String getLocation()            {
        return this.location;
    }

    public Date getDateStart()             {
        return this.dateStart;
    }

    public Date getDateEnd()               {
        return this.dateEnd;
    }

    public long getDuration()              {
        return this.duration;
    }

    public Date getCreateDate()            {
        return this.createDate;
    }

    public Date getLastModifiedDate()      {
        return this.lastModifiedDate;
    }

    public String getBusyStatus()          {
        return this.busyStatus;
    }

    public String getMeetingStatus()       {
        return this.meetingStatus;
    }

    public int getAllDayEvent()            {
        return this.allDayEvent;
    }

    public int getSequence()               {
            return this.sequence;
    }

    public Date getLastModified()          {
        return this.lastModified;
    }

    public char getState()                 {
        return this.state;
    }

    public String getHref()                {
        return this.href;
    }


    public void setId(String id)                               {
        this.id = id;
    }

    public void setUsername(String username)                   {
        this.username = username;
    }

    public void setSubject(String subject)                     {
        this.subject = subject;
    }

    public void setTextDescription(String textDescription)     {
        this.textDescription = textDescription;
    }

    public void setLocation(String location)                   {
        this.location = location;
    }

    public void setTimezoneId(int timezoneId)                  {
        this.timezoneId = timezoneId;
    }

    public void setDateStart(Date dateStart)                   {
        this.dateStart = dateStart;
    }

    public void setDateEnd(Date dateEnd)                       {
        this.dateEnd = dateEnd;
    }

    public void setDuration(long duration)                     {
        this.duration = duration;
    }

    public void setCreateDate(Date createDate)                 {
        this.createDate = createDate;
    }

    public void setLastModifiedDate(Date lastModifiedDate)     {
        this.lastModifiedDate = lastModifiedDate;
    }

    public void setBusyStatus(String busyStatus)               {
        this.busyStatus = busyStatus;
    }

    public void setMeetingStatus(String meetingStatus)         {
        this.meetingStatus = meetingStatus;
    }

    public void setAllDayEvent(int allDayEvent)                {
        this.allDayEvent = allDayEvent;
    }

    public void setSequence(int sequence)                      {
        this.sequence = sequence;
    }

    public void setLastModified(Date lastModified)             {
        this.lastModified = lastModified;
    }

    public void setState(char state)                           {
        this.state = state;
    }

    public void setHref(String href)                           {
        this.href = href;
    }

    /**
     * Set a Calendar properties from PocketPc content
     *
     *
     * @param id client id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public void getPPCalendar (String id, String content, Date t)
    throws CalendarParseException {

        String  tmz        = null ;
        String  dtStart    = null ;
        String  dtEnd      = null ;
        String  drt        = null ;
        String  ade        = null ;

        this.id            = id ;
        this.lastModified  = t   ;

        content = XmlParser.replaceTagDelimiter(content);

        try {

            XmlParser p = new XmlParser();

            tmz      = p.getXMLTagValue(content, TAG_PP_TIMEZONE_ID );
            dtStart  = p.getXMLTagValue(content, TAG_PP_DATE_START  );
            dtEnd    = p.getXMLTagValue(content, TAG_PP_DATE_END    );
            drt      = p.getXMLTagValue(content, TAG_PP_DURATION    );
            ade      = p.getXMLTagValue(content, TAG_ALL_DAY_EVENT  );

            dateStart     = p.webDavTagToDate (dtStart );
            dateEnd       = p.webDavTagToDate (dtEnd   );
            timezoneId    = Integer.parseInt  (tmz     );
            duration      = Long.parseLong    (drt     );
            allDayEvent   = Integer.parseInt  (ade     );

            subject       = p.getXMLTagValue (content, TAG_PP_SUBJECT        );
            location      = p.getXMLTagValue (content, TAG_PP_LOCATION       );
            textDescription
                = p.getXMLTagValue  (content, TAG_PP_NOTE                    );
            busyStatus    = p.getXMLTagValue (content, TAG_PP_BUSY_STATUS    );
            meetingStatus = p.getXMLTagValue (content, TAG_PP_MEETING_STATUS );

            if (EXCHANGE_OOF_STATE.equals(busyStatus)) {
                busyStatus = PP_OOF_STATE;
            }

        } catch (NumberFormatException e) {
            throw new CalendarParseException(e.getMessage());
        } catch (XmlParseException e) {
            throw new CalendarParseException(e.getMessage());
        }

    }

    public String toPP()
    throws CalendarParseException {

         String ppMsg = null;

         String dtStart = null ;
         String dtEnd   = null ;
         String drt     = null ;
         String bs      = null ;
         String ade     = null ;
         String tmz     = null;

         XmlParser p = new XmlParser();

         try {

             dtStart = p.dateToWebDavTag (dateStart   ) ;
             dtEnd   = p.dateToWebDavTag (dateEnd     ) ;
             drt     = String.valueOf    (duration    ) ;
             ade     = String.valueOf    (allDayEvent ) ;
             tmz     = String.valueOf    (timezoneId  ) ;

             if (EXCHANGE_OOF_STATE.equals(this.busyStatus)) {
                 bs = PP_OOF_STATE;
             } else {
                 bs = this.busyStatus;
             }

             ppMsg = MessageFormat.format(MSG_PP,
                new Object[] {
                                p.getString(subject         ) ,
                                p.getString(location        ) ,
                                p.getString(textDescription ) ,
                                p.getString(bs              ) ,
                                p.getString(meetingStatus   ) ,
                                p.getString(tmz             ) ,
                                p.getString(dtStart         ) ,
                                p.getString(dtEnd           ) ,
                                p.getString(drt             ) ,
                                p.getString(ade             )

                              }
             );

         } catch (Exception e) {
             throw new CalendarParseException ("Error compose calendar PocketPC: " +
                                               e.toString());
         }

         return ppMsg;

     }

}