/**
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */

package sync4j.exchange.items.contact.manager;

import java.io.IOException;

import sync4j.exchange.items.contact.dao.ContactDAO;
import sync4j.exchange.items.contact.model.Contact;
import sync4j.exchange.DataAccessException;

/*
 * This class defines methods to access contacts data
 * in exchange server datastore
 *
 * @author Fabio Maggi @ Funambol
 * @version $Id: ContactManager.java,v 1.2 2004/04/06 11:26:17 ste Exp $
 *
 **/
public class ContactManager {

    //------------------------------------------------------------- Constants

    //------------------------------------------------------------- Private Data

    ContactDAO cd = null ;

    //------------------------------------------------------------- Costructor

    public ContactManager(String exchangeServerHost ,
                          int    exchangeServerPort ,
                          String timezone)
        throws DataAccessException {

        this.cd = new ContactDAO (exchangeServerHost ,
                                  exchangeServerPort ,
                                  timezone);
    }

    //------------------------------------------------------------- Public methods

    /**
     * get all contacts
     *
     * @param username
     * @param credentials
     * @param sourceURI
     *
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getAllContacts(String   username    ,
                                    String   credentials ,
                                    String   sourceURI)
        throws DataAccessException {

        return this.cd.getContacts(username      ,
                                   credentials   ,
                                   new String[0] ,
                                   sourceURI);

    }

    /**
     * get contacts
     *
     * @param username
     * @param credentials
     * @param ids
     * @param sourceURI
     *
     * @return array of find contacts
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact[] getContacts(String   username    ,
                                 String   credentials ,
                                 String[] ids         ,
                                 String   sourceURI)
        throws DataAccessException {


        return this.cd.getContacts(username    ,
                                   credentials ,
                                   ids         ,
                                   sourceURI);

    }

    /**
     * get contact by key
     *
     * @param id
     * @param username
     * @param credentials
     *
     * @return find contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact getContactById(String username    ,
                                  String credentials ,
                                  String id          ,
                                  String sourceURI)
        throws DataAccessException {

        Contact[] contacts = null;

        contacts = this.cd.getContacts(username          ,
                                       credentials       ,
                                       new String[] {id} ,
                                       sourceURI);

        if (contacts == null || !(contacts.length > 0)) {
            return null;
        }

        return contacts[0];

    }

    /**
     * add contact
     *
     * @param contact
     * @param username
     * @param credentials
     * @param sourceURI
     *
     * @return new contact
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public Contact setContact(Contact contact    ,
                              String  username     ,
                              String  credentials ,
                              String  sourceURI
                              )
        throws DataAccessException {

        return this.cd.setContact(contact     ,
                                  username    ,
                                  credentials ,
                                  sourceURI);

    }

    /**
     * remove contact
     *
     * @param contact
     * @param username
     * @param principal
     * @param sourceURI
     *
     * @throws sync4j.exchange.util.DataAccessException
     **/
    public void removeContact(Contact contact   ,
                              String  username  ,
                              String  principal ,
                              String  sourceURI)
        throws DataAccessException {

        this.cd.removeContact(contact, username, principal, sourceURI);

    }

}