/**
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.contact.model;

import java.util.StringTokenizer;
import java.util.Date;
import java.text.MessageFormat;
import java.text.ParseException;

import sync4j.exchange.items.contact.ContactParseException;
import sync4j.exchange.xml.XmlParser;
import sync4j.exchange.xml.XmlParseException;

/**
 * @author Fabio Maggi @ Funambol
 *
 * @version $Id: Contact.java,v 1.3 2004/05/07 10:21:24 fabius Exp $
 **/

public class Contact {

    //--------------------------------------------------------------------- Constants

    private static final String KEY_ITEM_CLIENT_ENDTAG    = ";"                ;

    private static final String KEY_ITEM_CLIENT_EMAIL     = "EMAIL:"           ;
    private static final String KEY_ITEM_CLIENT_NAME      = "N:"               ;
    private static final String KEY_ITEM_CLIENT_ORG       = "ORG:"             ;
    private static final String KEY_ITEM_CLIENT_TITLE     = "TITLE:"           ;
    private static final String KEY_ITEM_CLIENT_TEL_FAX   = "TEL;FAX:"         ;
    private static final String KEY_ITEM_CLIENT_TEL_CELL  = "TEL;CELL:"        ;
    private static final String KEY_ITEM_CLIENT_TEL_VOICE = "TEL;HOME:"        ;

    private static final String TAG_PP_FIRST_NAME         = "first-name"       ;
    private static final String TAG_PP_LAST_NAME          = "last-name"        ;
    private static final String TAG_PP_MIDDLE_NAME        = "middle-name"      ;
    private static final String TAG_PP_TITLE              = "title"            ;
    private static final String TAG_PP_ORGANIZATION       = "organization"     ;

    private static final String TAG_PP_HOME_STREET        = "home-street"      ;
    private static final String TAG_PP_HOME_CITY          = "home-city"        ;
    private static final String TAG_PP_HOME_STATE         = "home-state"       ;
    private static final String TAG_PP_HOME_POSTAL_CODE   = "home-postalcode"  ;
    private static final String TAG_PP_HOME_COUNTRY       = "home-country"     ;

    private static final String TAG_PP_OFFICE_STREET      = "office-street"    ;
    private static final String TAG_PP_OFFICE_CITY        = "office-city"      ;
    private static final String TAG_PP_OFFICE_STATE       = "office-state"     ;
    private static final String TAG_PP_OFFICE_POSTAL_CODE = "office-postalcode";
    private static final String TAG_PP_OFFICE_COUNTRY     = "office-country"   ;

    private static final String TAG_PP_OTHER_STREET       = "other-street"     ;
    private static final String TAG_PP_OTHER_CITY         = "other-city"       ;
    private static final String TAG_PP_OTHER_STATE        = "other-state"      ;
    private static final String TAG_PP_OTHER_POSTAL_CODE  = "other-postalcode" ;
    private static final String TAG_PP_OTHER_COUNTRY      = "other-country"    ;

    private static final String TAG_PP_OFFICE_PHONE       = "office-phone"     ;
    private static final String TAG_PP_HOME_PHONE         = "home-phone"       ;
    private static final String TAG_PP_MOBILE_PHONE       = "mobile-phone"     ;
    private static final String TAG_PP_SECRETARY_PHONE    = "secretary-phone"  ;
    private static final String TAG_PP_OFFICE_PHONE2      = "office-phone2"    ;
    private static final String TAG_PP_OFFICE_FAX         = "office-fax"       ;
    private static final String TAG_PP_CAR_PHONE          = "car-phone"        ;
    private static final String TAG_PP_HOME_PHONE2        = "home-phone2"      ;
    private static final String TAG_PP_HOME_FAX           = "home-fax"         ;
    private static final String TAG_PP_PAGER              = "pager"            ;

    private static final String TAG_PP_BUSINESS_HOME_PAGE = "business-homepage";
    private static final String TAG_PP_NOTE               = "text-description" ;
    private static final String TAG_PP_DEPARTMENT         = "department"       ;
    private static final String TAG_PP_SPOUSE             = "spouse"           ;
    private static final String TAG_PP_SECRETARY          = "secretary"        ;
    private static final String TAG_PP_FILEAS             = "fileas"           ;

    private static final String TAG_PP_EMAIL1             = "email1"           ;
    private static final String TAG_PP_EMAIL1_NAME        = "email1-name"      ;
    private static final String TAG_PP_EMAIL2             = "email2"           ;
    private static final String TAG_PP_EMAIL2_NAME        = "email2-name"      ;
    private static final String TAG_PP_EMAIL3             = "email3"           ;
    private static final String TAG_PP_EMAIL3_NAME        = "email3-name"      ;

    private static final String MSG_VCARD                                      =
                    "BEGIN:VCARD\n"                                            +
                    "VERSION:2.1\n"                                            +
                    "REV:\n"                                                   +
                    "N:{0};{1};;;\n"                                           +
                    "ORG:{2};\n"                                               +
                    "TITLE:{3}\n"                                              +
                    "TEL;HOME:{4}\n"                                           +
                    "TEL;CELL:{5}\n"                                           +
                    "TEL;FAX:{6}\n"                                            +
                    "EMAIL:{7}\n"                                              +
                    "END:VCARD[lab@dh0337 contact]$"                           ;

    private static final String MSG_PP                                         =
                    "<![CDATA[\n"                                              +
                    "<first-name>{0}</first-name>\n"                           +
                    "<middle-name>{1}</middle-name>\n"                         +
                    "<last-name>{2}</last-name>\n"                             +
                    "<title>{3}</title>\n"                                     +
                    "<organization>{4}</organization>\n"                       +
                    "<home-street>{5}</home-street>\n"                         +
                    "<home-city>{6}</home-city>\n"                             +
                    "<home-state>{7}</home-state>\n"                           +
                    "<home-postalcode>{8}</home-postalcode>\n"                 +
                    "<home-country>{9}</home-country>\n"                       +
                    "<office-street>{10}</office-street>\n"                    +
                    "<office-city>{11}</office-city>\n"                        +
                    "<office-state>{12}</office-state>\n"                      +
                    "<office-postalcode>{13}</office-postalcode>\n"            +
                    "<office-country>{14}</office-country>\n"                  +
                    "<other-street>{15}</other-street>\n"                      +
                    "<other-city>{16}</other-city>\n"                          +
                    "<other-state>{17}</other-state>\n"                        +
                    "<other-postalcode>{18}</other-postalcode>\n"              +
                    "<other-country>{19}</other-country>\n"                    +
                    "<office-phone>{20}</office-phone>\n"                      +
                    "<home-phone>{21}</home-phone>\n"                          +
                    "<mobile-phone>{22}</mobile-phone>\n"                      +
                    "<secretary-phone>{23}</secretary-phone>\n"                +
                    "<office-phone2>{24}</office-phone2>\n"                    +
                    "<office-fax>{25}</office-fax>\n"                          +
                    "<car-phone>{26}</car-phone>\n"                            +
                    "<home-phone2>{27}</home-phone2>\n"                        +
                    "<home-fax>{28}</home-fax>\n"                              +
                    "<pager>{29}</pager>\n"                                    +
                    "<email1-name>{30}</email1-name>\n"                        +
                    "<email1>{31}</email1>\n"                                  +
                    "<email2-name>{32}</email2-name>\n"                        +
                    "<email2>{33}</email2>\n"                                  +
                    "<email3-name>{34}</email3-name>\n"                        +
                    "<email3>{35}</email3>\n"                                  +
                    "<business-homepage>{36}</business-homepage>\n"            +
                    "<text-description>{37}</text-description>\n"              +
                    "<department>{38}</department>\n"                          +
                    "<spouse>{39}</spouse>\n"                                  +
                    "<secretary>{40}</secretary>\n"                            +
                    "<fileas>{41}</fileas>\n"                                  +
                    "]]>"                                                      ;

    //--------------------------------------------------------------------- Private data

    private String   id                              = null ;

    private String   username                        = null ;

    private String   firstName                       = null ;
    private String   middleName                      = null ;
    private String   lastName                        = null ;

    private String   department                      = null ;
    private String   organization                    = null ;
    private String   office                          = null ;
    private String   title                           = null ;
    private String   profession                      = null ;
    private String   nickName                        = null ;
    private String   spouse                          = null ;
    private String   manager                         = null ;
    private String   secretary                       = null ;

    private String   homeStreet                      = null ;
    private String   homeCity                        = null ;
    private String   homeState                       = null ;
    private String   homePostalCode                  = null ;
    private String   homeCountry                     = null ;
    private String   homePhone                       = null ;
    private String   homeOtherPhone                  = null ;
    private String   homeFax                         = null ;

    private String   officeStreet                    = null ;
    private String   officeCity                      = null ;
    private String   officeState                     = null ;
    private String   officePostalCode                = null ;
    private String   officeCountry                   = null ;
    private String   officePhone                     = null ;
    private String   officeOtherPhone                = null ;
    private String   officeFax                       = null ;

    private String   otherStreet                     = null ;
    private String   otherCity                       = null ;
    private String   otherState                      = null ;
    private String   otherPostalCode                 = null ;
    private String   otherCountry                    = null ;
    private String   otherPhone                      = null ;
    private String   otherFax                        = null ;

    private String   callBackPhone                   = null ;
    private String   mobilePhone                     = null ;
    private String   mobileOtherPhone                = null ;
    private String   organizationPhone               = null ;
    private String   secretaryPhone                  = null ;

    private String   internationalIsdnNumber         = null ;
    private String   pager                           = null ;

    private String   emailAddress1                   = null ;
    private String   emailDescription1               = null ;
    private String   emailAddress2                   = null ;
    private String   emailDescription2               = null ;
    private String   emailAddress3                   = null ;
    private String   emailDescription3               = null ;

    private String   businessHomePage                = null ;
    private String   burl                            = null ;
    private String   note                            = null ;

    private String   fileas                          = null ;

    private String   href                            = null ;

    private Date     lastModified                    = null ;
    private char     state                           = ' '  ;

    //------------------------------------------------------------ Constructors

    /**
     * Creates a Contact
     *
     */
    public Contact () {

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     *
     */
    public Contact (String id) {

        this.id = id   ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param t client item timestamp
     *
     */
    public Contact (String id, Date t) {

        this.id            = id   ;
        this.lastModified  = t    ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param state client item state
     *
     */
    public Contact (String id, char state) {

        this.id      = id     ;
        this.state   = state  ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param state client item state
     *
     */
    public Contact (String   id    ,
                    String   href  ,
                    char     state ,
                    Date     lastModified) {

        this.id            = id            ;
        this.href          = href          ;
        this.state         = state         ;
        this.lastModified  = lastModified  ;

    }

    /**
     * Creates a Contact
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public Contact (String id, String content, Date t)
    throws ContactParseException {

        getPocketPcContact (id, content, t);

    }

    //----------------------------------------------------------------------- Public methods

    public String getId()                      {
        return this.id;
    }

    public String getUsername()                {
            return this.username;
    }

    public String getFirstName()               {
        return this.firstName;
    }

    public String getMiddleName()              {
        return this.middleName;
    }

    public String getLastName()                {
        return this.lastName;
    }

    public String getDepartment()              {
        return this.department;
    }

    public String getOffice()                  {
        return this.office;
    }

    public String getTitle()                   {
        return this.title;
    }

    public String getProfession()              {
        return this.profession;
    }

    public String getOrganization()            {
        return this.organization;
    }

    public String getNickName()                {
        return this.nickName;
    }

    public String getSpouse()                  {
        return this.spouse;
    }

    public String getManager()                 {
        return this.manager;
    }

    public String getSecretary()               {
        return this.secretary;
    }

    public String getHomeStreet()              {
        return this.homeStreet;
    }

    public String getHomeCity()                {
        return this.homeCity;
    }

    public String getHomeState()               {
        return this.homeState;
    }

    public String getHomePostalCode()          {
        return this.homePostalCode;
    }

    public String getHomeCountry()             {
        return this.homeCountry;
    }

    public String getHomePhone()               {
        return this.homePhone;
    }

    public String getHomeOtherPhone()          {
        return this.homeOtherPhone;
    }

    public String getHomeFax()                 {
        return this.homeFax;
    }

    public String getOfficeStreet()            {
        return this.officeStreet;
    }

    public String getOfficeCity()              {
        return this.officeCity;
    }

    public String getOfficeState()             {
        return this.officeState;
    }

    public String getOfficePostalCode()        {
        return this.officePostalCode;
    }

    public String getOfficeCountry()           {
        return this.officeCountry;
    }

    public String getOfficePhone()             {
        return this.officePhone;
    }

    public String getOfficeOtherPhone()        {
        return this.officeOtherPhone;
    }

    public String getOfficeFax()               {
        return this.officeFax;
    }

    public String getOtherStreet()             {
            return this.otherStreet;
    }

    public String getOtherCity()               {
        return this.otherCity;
    }

    public String getOtherState()              {
        return this.otherState;
    }

    public String getOtherPostalCode()         {
        return this.otherPostalCode;
    }

    public String getOtherCountry()            {
        return this.otherCountry;
    }

    public String getOtherPhone()              {
        return this.otherPhone;
    }

    public String getOtherFax()                {
        return this.otherFax;
    }

    public String getCallBackPhone()           {
        return this.callBackPhone;
    }

    public String getMobilePhone()             {
        return this.mobilePhone;
    }

    public String getMobileOtherPhone()        {
        return this.mobileOtherPhone;
    }

    public String getOrganizationPhone()       {
        return this.organizationPhone;
    }

    public String getSecretaryPhone()          {
        return this.secretaryPhone;
    }

    public String getInternationalIsdnNumber() {
        return this.internationalIsdnNumber;
    }

    public String getPager()                   {
        return this.pager;
    }

    public String getEmailAddress1()           {
        return this.emailAddress1;
    }

    public String getEmailDescription1()       {
        return this.emailDescription1;
    }

    public String getEmailAddress2()           {
        return this.emailAddress2;
    }

    public String getEmailDescription2()       {
        return this.emailDescription2;
    }

    public String getEmailAddress3()           {
        return this.emailAddress3;
    }

    public String getEmailDescription3()       {
        return this.emailDescription3;
    }

    public String getBusinessHomePage()        {
        return this.businessHomePage;
    }

    public String getBurl()                    {
        return this.burl;
    }

    public String getNote()                    {
        return this.note;
    }

    public String getFileas()                  {
        return this.fileas;
    }

    public Date getLastModified()              {
        return this.lastModified;
    }

    public char getState()                     {
        return this.state;
    }

    public String getHref()                    {
        return this.href;
    }


    public void setId(String id)                                           {
        this.id = id;
    }

    public void setUsername(String username)                               {
            this.username = username;
    }

    public void setFirstName(String firstName)                             {
        this.firstName = firstName;
    }

    public void setMiddleName(String middleName)                           {
        this.middleName = middleName;
    }

    public void setLastName(String lastName)                               {
        this.lastName = lastName;
    }

    public void setDepartment(String department)                           {
        this.department = department;
    }

    public void setOffice(String office)                                   {
        this.office = office;
    }

    public void setTitle(String title)                                     {
        this.title = title;
    }

    public void setProfession(String profession)                           {
            this.profession = profession;
    }

    public void setOrganization(String organization)                       {
        this.organization = organization;
    }

    public void setNickName(String nickName)                               {
        this.nickName = nickName;
    }

    public void setSpouse(String spouse)                                   {
        this.spouse = spouse;
    }

    public void setManager(String manager)                                 {
        this.manager = manager;
    }

    public void setSecretary(String secretary)                             {
        this.secretary = secretary;
    }

    public void setHomeStreet(String homeStreet)                           {
        this.homeStreet = homeStreet;
    }

    public void setHomeCity(String homeCity)                               {
        this.homeCity = homeCity;
    }

    public void setHomeState(String homeState)                             {
        this.homeState = homeState;
    }

    public void setHomePostalCode(String homePostalCode)                   {
        this.homePostalCode = homePostalCode;
    }

    public void setHomeCountry(String homeCountry)                         {
        this.homeCountry = homeCountry;
    }

    public void setHomePhone(String homePhone)                             {
        this.homePhone = homePhone;
    }

    public void setHomeOtherPhone(String homeOtherPhone)                   {
        this.homeOtherPhone = homeOtherPhone;
    }

    public void setHomeFax(String homeFax)                                 {
        this.homeFax = homeFax;
    }

    public void setOfficeStreet(String officeStreet)                       {
        this.officeStreet = officeStreet;
    }

    public void setOfficeCity(String officeCity)                           {
        this.officeCity = officeCity;
    }

    public void setOfficeState(String officeState)                         {
        this.officeState = officeState;
    }

    public void setOfficePostalCode(String officePostalCode)               {
        this.officePostalCode = officePostalCode;
    }

    public void setOfficeCountry(String officeCountry)                     {
        this.officeCountry = officeCountry;
    }

    public void setOfficePhone(String officePhone)                         {
        this.officePhone = officePhone;
    }

    public void setOfficeOtherPhone(String officeOtherPhone)               {
        this.officeOtherPhone = officeOtherPhone;
    }

    public void setOfficeFax(String officeFax)                             {
        this.officeFax = officeFax;
    }

    public void setOtherStreet(String otherStreet)                         {
        this.otherStreet = otherStreet;
    }

    public void setOtherCity(String otherCity)                             {
        this.otherCity = otherCity;
    }

    public void setOtherState(String otherState)                           {
        this.otherState = otherState;
    }

    public void setOtherPostalCode(String otherPostalCode)                 {
        this.otherPostalCode = otherPostalCode;
    }

    public void setOtherCountry(String otherCountry)                       {
        this.otherCountry = otherCountry;
    }

    public void setOtherPhone(String otherPhone)                           {
        this.otherPhone = otherPhone;
    }

    public void setOtherFax(String otherFax)                               {
        this.otherFax = otherFax;
    }

    public void setCallBackPhone(String callBackPhone)                     {
        this.callBackPhone = callBackPhone;
    }

    public void setMobilePhone(String mobilePhone)                         {
        this.mobilePhone = mobilePhone;
    }

    public void setMobileOtherPhone(String mobileOtherPhone)               {
        this.mobileOtherPhone = mobileOtherPhone;
    }

    public void setOrganizationPhone(String organizationPhone)             {
        this.organizationPhone = organizationPhone;
    }

    public void setSecretaryPhone(String secretaryPhone)                   {
        this.secretaryPhone = secretaryPhone;
    }

    public void setInternationalIsdnNumber(String internationalIsdnNumber) {
        this.internationalIsdnNumber = internationalIsdnNumber;
    }

    public void setPager(String pager)                                     {
        this.pager = pager;
    }

    public void setEmailAddress1(String emailAddress1)                     {
        this.emailAddress1 = emailAddress1;
    }

    public void setEmailDescription1(String emailDescription1)             {
        this.emailDescription1 = emailDescription1;
    }

    public void setEmailAddress2(String emailAddress2)                     {
        this.emailAddress2 = emailAddress2;
    }

    public void setEmailDescription2(String emailDescription2)             {
        this.emailDescription2 = emailDescription2;
    }

    public void setEmailAddress3(String emailAddress3)                     {
        this.emailAddress3 = emailAddress3;
    }

    public void setEmailDescription3(String emailDescription3)             {
        this.emailDescription3 = emailDescription3;
    }

    public void setBusinessHomePage(String businessHomePage)               {
        this.businessHomePage = businessHomePage;
    }

    public void setBurl(String burl)                                       {
        this.burl = burl;
    }

    public void setNote(String note)                                       {
        this.note = note;
    }

    public void setFileas(String fileas)                                   {
        this.fileas = fileas;
    }

    public void setLastModified(Date lastModified)                         {
        this.lastModified = lastModified;
    }

    public void setState(char state)                                       {
        this.state = state;
    }

    public void setHref(String href)                                       {
        this.href = href;
    }

    /**
     * Set a Contact properties from PocketPc content
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public void getPocketPcContact (String id, String content, Date t)
    throws ContactParseException {

       XmlParser p = new XmlParser();

       this.id                   = id ;
       this.lastModified         = t  ;

       content = XmlParser.replaceTagDelimiter(content);

       try {

           this.firstName            = p.getXMLTagValue(content,
                                                        TAG_PP_FIRST_NAME    ) ;
           this.lastName             = p.getXMLTagValue(content,
                                                        TAG_PP_LAST_NAME     ) ;
           this.middleName           = p.getXMLTagValue(content,
                                                        TAG_PP_MIDDLE_NAME   ) ;
           this.title                = p.getXMLTagValue(content,
                                                        TAG_PP_TITLE         ) ;
           this.organization         = p.getXMLTagValue(content,
                                                        TAG_PP_ORGANIZATION  ) ;

           this.homeStreet           = p.getXMLTagValue(content,
                                                        TAG_PP_HOME_STREET   ) ;
           this.homeCity             = p.getXMLTagValue(content,
                                                        TAG_PP_HOME_CITY     ) ;
           this.homeState            = p.getXMLTagValue(content,
                                                        TAG_PP_HOME_STATE    ) ;
           this.homePostalCode       = p.getXMLTagValue(content,
                                               TAG_PP_HOME_POSTAL_CODE       ) ;
           this.homeCountry         = p.getXMLTagValue(content,
                                               TAG_PP_HOME_COUNTRY           ) ;

           this.officeStreet         = p.getXMLTagValue(content,
                                                        TAG_PP_OFFICE_STREET ) ;
           this.officeCity           = p.getXMLTagValue(content,
                                                        TAG_PP_OFFICE_CITY   ) ;
           this.officeState          = p.getXMLTagValue(content,
                                                        TAG_PP_OFFICE_STATE  ) ;
           this.officePostalCode     = p.getXMLTagValue(content,
                                               TAG_PP_OFFICE_POSTAL_CODE     ) ;
           this.officeCountry        = p.getXMLTagValue(content,
                                               TAG_PP_OFFICE_COUNTRY         ) ;

           this.otherStreet          = p.getXMLTagValue(content,
                                                        TAG_PP_OTHER_STREET  ) ;
           this.otherCity            = p.getXMLTagValue(content,
                                                        TAG_PP_OTHER_CITY    ) ;
           this.otherState           = p.getXMLTagValue(content,
                                                        TAG_PP_OTHER_STATE   ) ;
           this.otherPostalCode      = p.getXMLTagValue(content,
                                               TAG_PP_OTHER_POSTAL_CODE      ) ;
           this.otherCountry         = p.getXMLTagValue(content,
                                               TAG_PP_OTHER_COUNTRY          ) ;

           this.officePhone          = p.getXMLTagValue(content,
                                                        TAG_PP_OFFICE_PHONE  ) ;
           this.homePhone            = p.getXMLTagValue(content,
                                                        TAG_PP_HOME_PHONE    ) ;
           this.mobilePhone          = p.getXMLTagValue(content,
                                                        TAG_PP_MOBILE_PHONE  ) ;
           this.secretaryPhone       = p.getXMLTagValue(content,
                                               TAG_PP_SECRETARY_PHONE        ) ;
           this.officeOtherPhone     = p.getXMLTagValue(content,
                                                        TAG_PP_OFFICE_PHONE2 ) ;
           this.officeFax            = p.getXMLTagValue(content,
                                                        TAG_PP_OFFICE_FAX    ) ;
           this.mobileOtherPhone     = p.getXMLTagValue(content,
                                                        TAG_PP_CAR_PHONE     ) ;
           this.homeOtherPhone       = p.getXMLTagValue(content,
                                                        TAG_PP_HOME_PHONE2   ) ;
           this.homeFax              = p.getXMLTagValue(content,
                                                        TAG_PP_HOME_FAX      ) ;
           this.pager                = p.getXMLTagValue(content,
                                                        TAG_PP_PAGER         ) ;

           this.businessHomePage     = p.getXMLTagValue(content,
                                               TAG_PP_BUSINESS_HOME_PAGE     ) ;
           this.note                 = p.getXMLTagValue(content,
                                                        TAG_PP_NOTE          ) ;
           this.department           = p.getXMLTagValue(content,
                                                        TAG_PP_DEPARTMENT    ) ;
           this.spouse               = p.getXMLTagValue(content,
                                                        TAG_PP_SPOUSE        ) ;
           this.secretary            = p.getXMLTagValue(content,
                                                        TAG_PP_SECRETARY     ) ;
           this.fileas               = p.getXMLTagValue(content,
                                                        TAG_PP_FILEAS        ) ;

           this.emailAddress1        = p.getXMLTagValue(content,
                                                        TAG_PP_EMAIL1        ) ;
           this.emailDescription1    = p.getXMLTagValue(content,
                                                        TAG_PP_EMAIL1_NAME   ) ;
           this.emailAddress2        = p.getXMLTagValue(content,
                                                        TAG_PP_EMAIL2        ) ;
           this.emailDescription2    = p.getXMLTagValue(content,
                                                        TAG_PP_EMAIL2_NAME   ) ;
           this.emailAddress3        = p.getXMLTagValue(content,
                                                        TAG_PP_EMAIL3        ) ;
           this.emailDescription3    = p.getXMLTagValue(content,
                                                        TAG_PP_EMAIL3_NAME   ) ;

        } catch (XmlParseException e) {
                    throw new ContactParseException ("Error compose contact vcard: " +
                                                      e.getMessage());
        }

     }


     public String toPP()
     throws ContactParseException {

         String ppMsg = null;

         XmlParser p = new XmlParser();

          try {

             ppMsg = MessageFormat.format(MSG_PP,
                 new Object[] {
                                p.getString(firstName         ) ,
                                p.getString(middleName        ) ,
                                p.getString(lastName          ) ,
                                p.getString(title             ) ,
                                p.getString(organization      ) ,
                                p.getString(homeStreet        ) ,
                                p.getString(homeCity          ) ,
                                p.getString(homeState         ) ,
                                p.getString(homePostalCode    ) ,
                                p.getString(homeCountry       ) ,
                                p.getString(officeStreet      ) ,
                                p.getString(officeCity        ) ,
                                p.getString(officeState       ) ,
                                p.getString(officePostalCode  ) ,
                                p.getString(officeCountry     ) ,
                                p.getString(otherStreet       ) ,
                                p.getString(otherCity         ) ,
                                p.getString(otherState        ) ,
                                p.getString(otherPostalCode   ) ,
                                p.getString(otherCountry      ) ,
                                p.getString(officePhone       ) ,
                                p.getString(homePhone         ) ,
                                p.getString(mobilePhone       ) ,
                                p.getString(secretaryPhone    ) ,
                                p.getString(officeOtherPhone  ) ,
                                p.getString(officeFax         ) ,
                                p.getString(mobileOtherPhone  ) ,
                                p.getString(homeOtherPhone    ) ,
                                p.getString(homeFax           ) ,
                                p.getString(pager             ) ,
                                p.getString(emailAddress1     ) ,
                                p.getString(emailDescription1 ) ,
                                p.getString(emailAddress2     ) ,
                                p.getString(emailDescription2 ) ,
                                p.getString(emailAddress3     ) ,
                                p.getString(emailDescription3 ) ,
                                p.getString(businessHomePage  ) ,
                                p.getString(note              ) ,
                                p.getString(department        ) ,
                                p.getString(spouse            ) ,
                                p.getString(secretary         ) ,
                                p.getString(fileas            )



                              }
             );

         } catch (Exception e) {
             throw new ContactParseException ("Error compose contact PocketPC: " +
                                               e.getMessage());
         }

         return ppMsg;

     }

}