/**
 * Copyright (C) 2003-2004 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package sync4j.exchange.items.task.model;

import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;

import java.util.Date;
import java.util.StringTokenizer;

import sync4j.exchange.items.task.TaskParseException;
import sync4j.exchange.xml.XmlParser;
import sync4j.exchange.xml.XmlParseException;

/**
 * @author Fabio Maggi @ Funambol
 *
 * @version $Id: Task.java,v 1.4 2004/05/07 10:21:24 fabius Exp $
 **/

public class Task {

    //--------------------------------------------------------------------- Constants

    private static final String KEY_ITEM_CLIENT_SUMMARY      = "SUMMARY:"     ;
    private static final String KEY_ITEM_CLIENT_DESCRIPTION  = "DESCRIPTION:" ;
    private static final String KEY_ITEM_CLIENT_DATE         = "DATE:"        ;

    private static final String TAG_PP_DATE                  = "date"         ;
    private static final String TAG_PP_SUBJECT               = "subject"      ;
    private static final String TAG_PP_NOTE
        = "text-description"                                                  ;

    private static final String MSG_VCARD                =
          "BEGIN:VCALENDAR\n"                            +
          "VERSION:1.0\n"                                +
          "BEGIN:VEVENT\n"                               +
          "UID:1074800413959\n"                          +
          "DESCRIPTION:{0}\n"                            +
          "SUMMARY:{1}\n"                                +
          "DATE:{2}\n"                                   +
          "X-EPOCAGENDAENTRYTYPE:exchange\n"             +
          "CATEGORIES:exchange\n"                        +
          "END:VEVENT"                                   ;

    private static final String MSG_PP                   =

          "<![CDATA[\n"                                  +
          "<subject>{0}</subject>\n"                     +
          "<text-description>{1}</text-description>\n"   +
          "<date>{2}</date>\n"                           +
          "]]>"                                          ;

    //--------------------------------------------------------------------- Private data

    private String     id                  = null  ;

    private String     username            = null  ;

    private String     subject             = null  ;

    private String     textDescription     = null  ;

    private Date       date                = null  ;

    private Date       createDate          = null  ;

    private String     href                = null  ;

    private Date       lastModified        = null  ;
    private char       state               = ' '   ;

    //--------------------------------------------------------------------- Constructors

    /**
     * Creates a Task
     *
     */
    public Task () {

    }

    /**
     * Creates a Task
     *
     *
     * @param id client item id
     *
     */
    public Task (String id) {

        this.id = id ;

    }

    /**
     * Creates a Task
     *
     *
     * @param id client item id
     * @param t client item timestamp
     *
     */
    public Task (String id, Date t)
    throws TaskParseException {

        this.id            = id ;
        this.lastModified  = t  ;

    }

    /**
     * Creates a Task
     *
     * @param id client item id
     * @param state client item state
     *
     */
    public Task (String id, char state) {

        this.id      = id     ;
        this.state   = state  ;

    }

    /**
     * Creates a Task
     *
     * @param id
     * @param href
     * @param state
     * @param lastModified
     *
     */
    public Task (String     id    ,
                 String     href  ,
                 char       state ,
                 Date       lastModified) {

        this.id            = id            ;
        this.href          = href          ;
        this.state         = state         ;
        this.lastModified  = lastModified  ;

    }

    /**
     * Creates a Task
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public Task (String id, String content, Date t)
    throws TaskParseException {

        getPPTask      (id, content, t);

    }

    //----------------------------------------------------------------------- Public methods


    public String getId()                  {
        return this.id;
    }

    public String getUsername()            {
            return this.username;
    }

    public String getSubject()             {
        return this.subject;
    }

    public String getTextDescription()     {
        return this.textDescription;
    }

    public Date getDate()                  {
        return this.date;
    }

    public String getHref()                {
        return this.href;
    }

    public Date getLastModified()          {
        return this.lastModified;
    }

    public char getState()                 {
        return this.state;
    }


    public void setId(String id)                               {
        this.id = id;
    }

    public void setUsername(String username)                   {
        this.username = username;
    }

    public void setSubject(String subject)                     {
        this.subject = subject;
    }

    public void setTextDescription(String textDescription)     {
        this.textDescription = textDescription;
    }

    public void setDate(Date date)                             {
        this.date = date;
    }

    public void setHref(String href)                           {
        this.href = href;
    }

    public void setLastModified(Date lastModified)             {
        this.lastModified = lastModified;
    }

    public void setState(char state)                           {
        this.state = state;
    }

    /**
     * Set a Task properties from PocketPc content
     *
     *
     * @param id client item id
     * @param content client item content
     * @param t client item timestamp
     *
     */
    public void getPPTask (String id, String content, Date t)
    throws TaskParseException {

        String dt = null ;

        this.id            = id ;
        this.lastModified  = t   ;

        content = XmlParser.replaceTagDelimiter(content);

        try {

            XmlParser p = new XmlParser();

            dt = p.getXMLTagValue(content, TAG_PP_DATE );

            this.date    = p.webDavTagToDate (dt                      );
            this.subject = p.getXMLTagValue  (content, TAG_PP_SUBJECT );

            this.textDescription
                = p.getXMLTagValue  (content, TAG_PP_NOTE);

        } catch (NumberFormatException e) {
            throw new TaskParseException(e.getMessage());
        } catch (XmlParseException e) {
            throw new TaskParseException(e.getMessage());
        }

    }


    public String toPP()
    throws TaskParseException {

         String ppMsg = null;

         String date  = null ;

         XmlParser p = new XmlParser();

         try {

             date = p.dateToWebDavTag (this.date) ;

             ppMsg = MessageFormat.format(MSG_PP,
                new Object[] {
                                p.getString(subject         ) ,
                                p.getString(textDescription ) ,
                                p.getString(date            )

                              }
             );

         } catch (Exception e) {
             throw new TaskParseException ("Error compose calendar PocketPC: " +
                                           e.getMessage());
         }

         return ppMsg;

     }

}